@@ -11,12 +11,19 @@
 #include <queue>
 #include <functional>
 
+class Task
+{
+public:
+    std::function<void()> func;
+    unsigned char* buffer;
+};
+
 class WorkerThread
 {
 public:
     std::mutex m;
     std::condition_variable cv;
-    std::queue<std::function<void()>> tasks = {};
+    std::queue<Task> tasks = {};
     volatile bool running;
 	std::thread t;
 };
@@ -29,7 +36,19 @@ typedef struct
     WorkerThread *worker;
 } DynamsoftMrzReader;
 
-static int DynamsoftMrzReader_clear(DynamsoftMrzReader *self)
+void clearTasks(DynamsoftMrzReader *self)
+{
+    if (self->worker->tasks.size() > 0)
+    {
+        for (int i = 0; i < self->worker->tasks.size(); i++)
+        {
+            free(self->worker->tasks.front().buffer);
+            self->worker->tasks.pop();
+        }
+    }
+}
+
+void clear(DynamsoftMrzReader *self)
 {
     if (self->callback)
     {
@@ -37,21 +56,26 @@ static int DynamsoftMrzReader_clear(DynamsoftMrzReader *self)
         self->callback = NULL;
     }
 
-    if (self->handler)
-    {
-        DLR_DestroyInstance(self->handler);
-        self->handler = NULL;
-    }
-
     if (self->worker)
     {
         self->worker->running = false;
+        clearTasks(self);
         self->worker->cv.notify_one();
         self->worker->t.join();
         delete self->worker;
         self->worker = NULL;
         printf("Quit native thread.\n");
     }
+}
+
+static int DynamsoftMrzReader_clear(DynamsoftMrzReader *self)
+{
+    clear(self);
+    if (self->handler)
+    {
+        DLR_DestroyInstance(self->handler);
+        self->handler = NULL;
+    }
 
     return 0;
 }
@@ -260,7 +284,10 @@ void scan(DynamsoftMrzReader *self, unsigned char *buffer, int width, int height
     }
 
     free(buffer);
-    onResultReady(self);
+    if (self->callback)
+    {
+        onResultReady(self);
+    }
 }
 
 /**
@@ -274,7 +301,7 @@ static PyObject *decodeMatAsync(PyObject *obj, PyObject *args)
     DynamsoftMrzReader *self = (DynamsoftMrzReader *)obj;
     PyObject *o;
     if (!PyArg_ParseTuple(args, "O", &o))
-        return NULL;
+        Py_BuildValue("i", -1);
 
     Py_buffer *view;
     int nd;
@@ -282,7 +309,7 @@ static PyObject *decodeMatAsync(PyObject *obj, PyObject *args)
     if (memoryview == NULL)
     {
         PyErr_Clear();
-        return NULL;
+        Py_BuildValue("i", -1);
     }
 
     view = PyMemoryView_GET_BUFFER(memoryview);
@@ -312,13 +339,12 @@ static PyObject *decodeMatAsync(PyObject *obj, PyObject *args)
     memcpy(data, buffer, len);
 
     std::unique_lock<std::mutex> lk(self->worker->m);
-    if (self->worker->tasks.size() > 1)
-    {
-        std::queue<std::function<void()>> empty = {};
-        std::swap(self->worker->tasks, empty);
-    }
+    clearTasks(self);
     std::function<void()> task_function = std::bind(scan, self, data, width, height, stride, format, len);
-    self->worker->tasks.push(task_function);
+    Task task;
+    task.func = task_function;
+    task.buffer = data;
+    self->worker->tasks.push(task);
     self->worker->cv.notify_one();
     lk.unlock();
     
@@ -329,7 +355,7 @@ static PyObject *decodeMatAsync(PyObject *obj, PyObject *args)
 /**
  * Load MRZ configuration file.
  *
- * @param string filename
+ * @param string template content
  *
  * @return loading status
  */
@@ -337,14 +363,14 @@ static PyObject *loadModel(PyObject *obj, PyObject *args)
 {
     DynamsoftMrzReader *self = (DynamsoftMrzReader *)obj;
 
-    char *pFileName; // File name
-    if (!PyArg_ParseTuple(args, "s", &pFileName))
+    char *settings; // File name
+    if (!PyArg_ParseTuple(args, "s", &settings))
     {
-        return NULL;
+        Py_BuildValue("i", -1);
     }
 
     char errorMsgBuffer[512];
-    int ret = DLR_AppendSettingsFromFile(self->handler, pFileName, errorMsgBuffer, 512);
+    int ret = DLR_AppendSettingsFromString(self->handler, settings, errorMsgBuffer, 512);
     printf("Load MRZ model: %s\n", errorMsgBuffer);
 
     return Py_BuildValue("i", ret);
@@ -358,7 +384,11 @@ void run(DynamsoftMrzReader *self)
         std::unique_lock<std::mutex> lk(self->worker->m);
         self->worker->cv.wait(lk, [&]
                               { return !self->worker->tasks.empty() || !self->worker->running; });
-        task = std::move(self->worker->tasks.front());
+        if (!self->worker->running)
+		{
+			break;
+		}
+        task = std::move(self->worker->tasks.front().func);
         self->worker->tasks.pop();
         lk.unlock();
 
@@ -367,7 +397,7 @@ void run(DynamsoftMrzReader *self)
 }
 
 /**
- * Register callback function to receive barcode decoding result asynchronously.
+ * Register callback function to receive MRZ decoding result asynchronously.
  */
 static PyObject *addAsyncListener(PyObject *obj, PyObject *args)
 {
@@ -376,13 +406,13 @@ static PyObject *addAsyncListener(PyObject *obj, PyObject *args)
     PyObject *callback = NULL;
     if (!PyArg_ParseTuple(args, "O", &callback))
     {
-        return NULL;
+        Py_BuildValue("i", -1);
     }
 
     if (!PyCallable_Check(callback))
     {
         PyErr_SetString(PyExc_TypeError, "parameter must be callable");
-        return NULL;
+        Py_BuildValue("i", -1);
     }
     else
     {
@@ -402,12 +432,23 @@ static PyObject *addAsyncListener(PyObject *obj, PyObject *args)
     return Py_BuildValue("i", 0);
 }
 
+/**
+ * Clear native thread and tasks.
+ */
+static PyObject *clearAsyncListener(PyObject *obj, PyObject *args)
+{
+    DynamsoftMrzReader *self = (DynamsoftMrzReader *)obj;
+    clear(self);
+    return Py_BuildValue("i", 0);
+}
+
 static PyMethodDef instance_methods[] = {
     {"decodeFile", decodeFile, METH_VARARGS, NULL},
     {"decodeMat", decodeMat, METH_VARARGS, NULL},
     {"loadModel", loadModel, METH_VARARGS, NULL},
     {"addAsyncListener", addAsyncListener, METH_VARARGS, NULL},
     {"decodeMatAsync", decodeMatAsync, METH_VARARGS, NULL},
+    {"clearAsyncListener", clearAsyncListener, METH_VARARGS, NULL},
     {NULL, NULL, 0, NULL}};
 
 static PyTypeObject DynamsoftMrzReaderType = {
